"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# pylint: disable=missing-docstring,old-style-class,protected-access
# pylint: disable=attribute-defined-outside-init,too-few-public-methods,unused-argument

import json
import os
import unittest
import mock

from icetea_lib.ResourceProvider.ResourceProvider import ResourceProvider
from icetea_lib.ResourceProvider.exceptions import ResourceInitError
from icetea_lib import LogManager
from icetea_lib.ResourceProvider.Allocators.exceptions import AllocationError


class MockLogger:
    def __init__(self):
        pass

    def info(self, message):
        pass

    def error(self, message):
        pass

    def debug(self, message, content=None):
        pass

    def exception(self, message):
        pass


class MockAllocator:
    def __init__(self, thing1, thing2, thing3):
        self.allocate_calls = 0

    def reset_logging(self):
        pass

    def allocate(self, *args, **kwargs):
        self.allocate_calls = self.allocate_calls+1
        if self.allocate_calls == 1:
            raise AllocationError
        else:
            pass

    def cleanup(self):
        pass

    def release(self, dut=None):
        pass


class MockDut:
    def __init__(self):
        pass

    def close_dut(self):
        pass

    def close_connection(self):
        pass

    def release(self):
        pass

    def print_info(self):
        pass


class MockArgs:
    def __init__(self):
        self.parallel_flash = True
        self.allocator = "TestAllocator"
        self.list = False
        self.listsuites = False
        self.allocator_cfg = None


@mock.patch("icetea_lib.ResourceProvider.ResourceProvider.LogManager", spec=LogManager)
@mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.LocalAllocator.get_resourceprovider_logger")
class RPTestcase(unittest.TestCase):

    def test_init(self, mock_rplogger_get, mock_logman):
        mock_logman.get_resourceprovider_logger = mock.MagicMock(return_value=MockLogger())
        self.res_pro = ResourceProvider(MockArgs())
        mock_logman.get_resourceprovider_logger.assert_called_once_with("ResourceProvider",
                                                                        "RSP", True)

    def test_init_with_no_file_logging(self, mock_rplogger_get, mock_logman):
        mock_logman.get_resourceprovider_logger = mock.MagicMock(return_value=MockLogger())
        mock_arguments = MockArgs()
        mock_arguments.list = True
        self.res_pro = ResourceProvider(mock_arguments)
        mock_logman.get_resourceprovider_logger.assert_called_once_with("ResourceProvider",
                                                                        "RSP", False)

    def test_init_with_list(self, mock_rplogger_get, mock_logman):
        mock_logman.get_resourceprovider_logger = mock.MagicMock(return_value=MockLogger())
        args = MockArgs()
        args.list = True
        self.res_pro = ResourceProvider(args)
        mock_logman.get_resourceprovider_logger.assert_called_once_with("ResourceProvider",
                                                                        "RSP", False)

    def test_allocate_duts_errors(self, mock_rplogger_get, mock_logman):
        mock_logman.get_resourceprovider_logger = mock.MagicMock(return_value=MockLogger())
        self.res_pro = ResourceProvider(MockArgs())

        mock_resconf = mock.MagicMock()
        mock_pluginmanager = mock.MagicMock()
        mock_pluginmanager.get_allocator = mock.MagicMock(return_value=MockAllocator)
        self.res_pro.set_pluginmanager(mock_pluginmanager)

        mock_resconf.count_hardware = mock.MagicMock(return_value=0)
        mock_resconf.get_dut_configuration = mock.MagicMock(return_value=[])
        mock_resconf.count_duts = mock.MagicMock(return_value=0)
        self.res_pro._duts = [MockDut()]
        self.res_pro._resource_configuration = mock_resconf
        # Test raise when allocation fails
        with self.assertRaises(ResourceInitError):
            self.res_pro.allocate_duts(mock_resconf)

    def test_allocate_duts_success(self, mock_rplogger_get, mock_logman):
        mock_logman.get_resourceprovider_logger = mock.MagicMock(return_value=MockLogger())

        self.res_pro = ResourceProvider(MockArgs())

        mock_resconf = mock.MagicMock()
        mock_resconf.count_hardware = mock.MagicMock(return_value=1)
        mock_resconf.get_dut_configuration = mock.MagicMock(return_value=[mock.MagicMock()])
        mock_resconf.count_duts = mock.MagicMock(return_value=1)
        self.res_pro._duts = [MockDut()]
        self.res_pro._resource_configuration = mock_resconf
        self.res_pro.allocator = mock.MagicMock()
        self.res_pro.allocator.allocate = mock.MagicMock()
        self.res_pro.allocate_duts(mock_resconf)
        self.res_pro.allocator.allocate.assert_called_once_with(mock_resconf,
                                                                args=self.res_pro.args)

    def test_allocator_get(self, mock_rplogger_get, mock_logman):
        mock_logman.get_resourceprovider_logger = mock.MagicMock(return_value=MockLogger())
        m_args = MockArgs()
        mock_resconf = mock.MagicMock()
        mock_resconf.count_hardware = mock.MagicMock(return_value=1)
        mock_resconf.get_dut_configuration = mock.MagicMock(return_value=[mock.MagicMock()])
        mock_resconf.count_duts = mock.MagicMock(return_value=1)
        self.res_pro = ResourceProvider(m_args)
        self.res_pro._resource_configuration = mock_resconf
        mock_pluginmanager = mock.MagicMock()
        self.res_pro.set_pluginmanager(mock_pluginmanager)
        mock_allocator = mock.MagicMock()
        mock_pluginmanager.get_allocator = mock.MagicMock(side_effect=[mock_allocator, None])
        self.res_pro.allocate_duts(mock_resconf)
        mock_allocator.assert_called_once_with(m_args, None, dict())

        self.res_pro.allocator = None
        with self.assertRaises(ResourceInitError):
            self.res_pro.allocate_duts(mock_resconf)

    def test_config_file_reading(self, mock_rplogger_get, mock_logman):
        mock_logman.get_resourceprovider_logger = mock.MagicMock(return_value=MockLogger())
        filepath = os.path.abspath(os.path.join(__file__, os.path.pardir, "tests",
                                                "allocator_config.json"))
        self.res_pro = ResourceProvider(MockArgs())

        with open(filepath, "r") as cfg_file:
            test_data = json.load(cfg_file)

        self.res_pro = ResourceProvider(MockArgs())
        retval = self.res_pro._read_allocator_config("testallocator", filepath)
        self.assertEquals(retval, test_data.get("testallocator"))

    @mock.patch("icetea_lib.ResourceProvider.ResourceProvider.json")
    def test_config_file_errors(self, mock_rplogger_get, mock_logman, mocked_json):
        mock_logman.get_resourceprovider_logger = mock.MagicMock(return_value=MockLogger())
        self.res_pro = ResourceProvider(MockArgs())
        with self.assertRaises(ResourceInitError):
            self.res_pro._read_allocator_config("generic", "does_not_exist")
        with self.assertRaises(ResourceInitError):
            not_a_file = os.path.abspath(os.path.join(__file__, os.path.pardir, "tests"))
            self.res_pro._read_allocator_config("generic", not_a_file)
        with self.assertRaises(ResourceInitError):
            no_config_here = os.path.abspath(os.path.join(__file__, os.path.pardir, "suites",
                                                          "dummy_suite.json"))
            self.res_pro._read_allocator_config("generic", no_config_here)

        with self.assertRaises(ResourceInitError):
            mocked_json.load = mock.MagicMock()
            mocked_json.load.side_effect = [ValueError]
            filepath = os.path.abspath(os.path.join(__file__, os.path.pardir, "tests",
                                                    "allocator_config.json"))
            self.res_pro._read_allocator_config("testallocator", filepath)

    def tearDown(self):
        self.res_pro.cleanup()
        self.res_pro.__metaclass__._instances.clear()


if __name__ == '__main__':
    unittest.main()



# pylint: disable=missing-docstring,unnecessary-lambda

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import random
import unittest
import tempfile
from icetea_lib.Randomize.randomize import Randomize


class RandomizeTestcase(unittest.TestCase):
    def test_full_args(self):
        self.assertGreaterEqual(Randomize.random_integer(7, 3).value, 3)
        self.assertLessEqual(Randomize.random_integer(7, 3).value, 7)

        self.assertIn(Randomize.random_list_elem(['a', 'bb', 'cc']).value, ['a', 'bb', 'cc'])

        self.assertGreaterEqual(len(Randomize.random_string(7,
                                                            3,
                                                            lambda x: random.choice(x), x='e34r')),
                                3)
        self.assertLessEqual(len(Randomize.random_string(7,
                                                         3,
                                                         lambda x: random.choice(x), x='e34r')),
                             7)
        self.assertIn(Randomize.random_string(chars=["aa", "bb", "ceedee"]).value,
                      ["a", "b", "c", "d", "e"])

        self.assertIn(Randomize.random_array_elem(['a', 'bb', 'cc']).value, [['a'], ['bb'], ['cc']])

        self.assertGreaterEqual(len(Randomize.random_string_array(9,
                                                                  3,
                                                                  7,
                                                                  2,
                                                                  lambda x: random.choice(x),
                                                                  x='e34r')),
                                3)
        self.assertLessEqual(len(Randomize.random_string_array(9,
                                                               3,
                                                               7,
                                                               2,
                                                               lambda x: random.choice(x),
                                                               x='e34r')),
                             9)

    def test_chars_not_str(self):
        with self.assertRaises(ValueError):
            Randomize.random_string(7, 3, chars=6)

    def test_chars_not_list(self):
        with self.assertRaises(TypeError):
            Randomize.random_list_elem([6])

    def test_random_str_lst_chars_no_str(self):  # pylint: disable=invalid-name
        with self.assertRaises(TypeError):
            Randomize.random_string(2, 1, chars=["a", "abc", 1])

    def test_random_integer_add(self):
        i = Randomize.random_integer(6)
        self.assertTrue(i + 6 == i.value + 6)
        self.assertTrue(6 + i == i.value + 6)

    def test_random_string_add(self):
        rand_str = Randomize.random_string(5)
        value = rand_str.value
        self.assertEqual(rand_str + ' hello', value + ' hello')
        self.assertEqual('hello ' + rand_str, 'hello ' + value)

    def test_random_string_array_add(self):
        rand_str = Randomize.random_string_array(5, 3)
        value = rand_str.value
        self.assertEqual(rand_str + ['world'], value + ['world'])
        self.assertEqual(['world'] + rand_str, ['world'] + value)

    def test_random_integer_iadd(self):
        rand_integer = Randomize.random_integer(6)
        value = rand_integer.value

        rand_integer += 6
        value += 6

        self.assertEqual(rand_integer.value, value)

    def test_random_integer_repr(self):
        rand_integer = Randomize.random_integer(6)
        self.assertEqual("%s" % rand_integer, str(rand_integer.value))

    def test_random_string_repr(self):
        rand_str = Randomize.random_string(5)
        self.assertEqual("%s" % rand_str, str(rand_str.value))

    def test_random_string_array_repr(self):
        rand_str_ar = Randomize.random_string_array(5, 3)
        self.assertEqual("%s" % rand_str_ar, str(rand_str_ar.value))

    def test_random_string_iter(self):
        for elem in Randomize.random_string(7, 3):
            self.assertTrue(isinstance(elem, str))

    def test_random_string_array_iter(self):
        for elem in Randomize.random_string_array(7, 3):
            self.assertTrue(isinstance(elem, str))

    def test_random_string_get_item(self):
        rand_str = Randomize.random_string(6)
        value = rand_str.value
        self.assertEqual(rand_str[0], value[0])

    def test_random_string_array_get_item(self):  # pylint: disable=invalid-name
        rand_str_ar = Randomize.random_string_array(6, 3)
        value = rand_str_ar.value
        self.assertEqual(rand_str_ar[0], value[0])

    def test_random_string_str(self):
        s_str = Randomize.random_string(6)
        self.assertEqual(str(s_str), s_str.value)

    def test_reproduce(self):
        seed = Randomize.random_integer(20, 3).value

        def user_func(seed):
            random.seed(seed)
            return random.randint(5, 15), random.randint(15, 25)

        rep1, rep2 = user_func(seed)
        rep3, rep4 = user_func(seed)

        self.assertEqual(rep1, rep3)
        self.assertEqual(rep2, rep4)

    def test_store_load(self):
        s_int = Randomize.random_integer(8, 4)
        s_str = Randomize.random_string(8, 4, lambda x: random.choice(x), x='e34r')
        s_str_a = Randomize.random_string_array(10, 6, 7, 4)

        temp_file = tempfile.NamedTemporaryFile(delete=False)
        s_int.store(temp_file.name)
        self.assertEqual(s_int.value, s_int.load(temp_file.name).value)
        temp_file.close()

        temp_file = tempfile.NamedTemporaryFile(delete=False)
        s_str.store(temp_file.name)
        self.assertEqual(s_str.value, s_str.load(temp_file.name).value)
        temp_file.close()

        temp_file = tempfile.NamedTemporaryFile(delete=False)
        s_str_a.store(temp_file.name)
        self.assertEqual(s_str_a.value, s_str_a.load(temp_file.name).value)
        temp_file.close()



# pylint: disable=too-many-arguments
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Logger with helpers to enable logging in Bench.
"""

import icetea_lib.LogManager as LogManager


class Logger(object):
    """
    This Mixer provide public logger property for everybody usage.
    """
    def __init__(self, **kwargs):
        super(Logger, self).__init__(**kwargs)
        self.__logger = LogManager.get_dummy_logger()

    def get_logger(self):
        """
        Getter for the logger.

        :return: Logger
        """
        return self.__logger

    def init_logger(self, test_name, verbose, silent, color, disable_log_truncate):
        """
        This function is called from Bench right after test is started.
        """
        LogManager.init_testcase_logging(test_name, verbose,
                                         silent, color,
                                         not disable_log_truncate)
        self.__logger = LogManager.get_bench_logger()

    def set_logger(self, value):
        """
        Setter for logger.
        """
        self.__logger = value



# pylint: disable=missing-docstring,too-many-statements

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import argparse
import unittest
import os

from icetea_lib.build.build import Build
from icetea_lib.DeviceConnectors.DutInformation import DutInformation
from icetea_lib.Result import Result
from icetea_lib.ResultList import ResultList


class ResultListTestcase(unittest.TestCase):

    def setUp(self):
        self.args_tc = argparse.Namespace(
            available=False, version=False, bin=None, binary=False, channel=None,
            clean=False, cloud=False, component=False, device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False, iface=None, kill_putty=False, list=False,
            listsuites=False, log='./log', my_duts=None, nobuf=None, pause_when_external_dut=False,
            putty=False, reset=False, silent=True, skip_case=False,
            skip_rampdown=False, skip_rampup=False, platform=None,
            status=False, suite=None, tc="test_cmdline", tc_cfg=None,
            tcdir="examples", testtype=False, type="process",
            subtype=None, use_sniffer=False, valgrind=False,
            valgrind_tool=None, verbose=False, repeat=0, feature=None,
            suitedir="./test/suites", forceflash_once=True, forceflash=False,
            stop_on_failure=False, json=False)

    def test_append(self):
        # Test append for single Result
        rlist = ResultList()
        result1 = Result()
        rlist.append(result1)
        self.assertListEqual(rlist.data, [result1])

        # Test append for ResultList
        result2 = Result()
        rlist2 = ResultList()
        rlist2.append(result2)
        rlist.append(rlist2)
        self.assertListEqual(rlist.data, [result1, result2])

        # Test append TypeError
        with self.assertRaises(TypeError):
            rlist.append(["test"])

    def test_get_verdict(self):
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        reslist = ResultList()
        reslist.append(res)
        self.assertEquals(reslist.get_verdict(), "pass")
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=10)
        reslist.append(res2)
        self.assertEquals(reslist.get_verdict(), "fail")
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 1)
        reslist = ResultList()
        reslist.append(res3)
        self.assertEquals(reslist.get_verdict(), "inconclusive")
        reslist.append(res2)
        self.assertEquals(reslist.get_verdict(), "fail")

    def test_get_summary(self):
        expected = {"count": 3,
                    "pass": 1,
                    "fail": 1,
                    "skip": 0,
                    "inconclusive": 1,
                    "retries": 1,
                    "duration": 10}
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 5)
        res3.retries_left = 1
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        self.assertDictEqual(resultlist.get_summary(), expected)

    def test_inconc_count(self):
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 5)
        res3.retries_left = 1
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        self.assertEqual(resultlist.inconclusive_count(), 1)
        res4 = Result()
        res4.set_verdict("inconclusive", 4, 5)
        resultlist.append(res4)
        self.assertEqual(resultlist.inconclusive_count(), 2)

    def test_pass_count(self):
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 5)
        res3.retries_left = 1
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        self.assertEqual(resultlist.success_count(), 1)
        res4 = Result()
        res4.set_verdict("pass", 4, 5)
        resultlist.append(res4)
        self.assertEqual(resultlist.success_count(), 2)

    def test_fail_count(self):
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 5)
        res3.retries_left = 1
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        self.assertEqual(resultlist.failure_count(), 1)
        res4 = Result()
        res4.set_verdict("fail", 4, 5)
        resultlist.append(res4)
        self.assertEqual(resultlist.failure_count(), 2)

    def test_retries_count(self):
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 5)
        res3.retries_left = 1
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        self.assertEqual(resultlist.retry_count(), 1)
        res4 = Result()
        res4.set_verdict("inconclusive", 4, 5)
        resultlist.append(res4)
        self.assertEqual(resultlist.retry_count(), 1)

    def test_clean_fails(self):
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        res.set_verdict(verdict="pass", retcode=0, duration=0)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 5)
        res3.retries_left = 1
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        res4 = Result()
        res4.set_verdict("inconclusive", 4, 5)
        resultlist.append(res4)
        self.assertTrue(resultlist.clean_fails())

        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        res.set_verdict(verdict="pass", retcode=0, duration=0)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res2.retries_left = 1
        res3 = Result()
        res3.set_verdict("fail", 4, 5)
        res3.retries_left = 0
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        res4 = Result()
        res4.set_verdict("inconclusive", 4, 5)
        resultlist.append(res4)
        self.assertTrue(resultlist.clean_fails())

        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        res.set_verdict(verdict="pass", retcode=0, duration=0)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res2.retries_left = 1
        res3 = Result()
        res3.set_verdict("pass", 4, 5)
        res3.retries_left = 0
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        res4 = Result()
        res4.set_verdict("inconclusive", 4, 5)
        resultlist.append(res4)
        self.assertFalse(resultlist.clean_fails())

    def test_clean_inconcs(self):
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        res.set_verdict(verdict="pass", retcode=0, duration=0)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 5)
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        self.assertTrue(resultlist.clean_inconcs())

        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        res.set_verdict(verdict="pass", retcode=0, duration=0)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res2.retries_left = 1
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 5)
        res3.retries_left = 1
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        res4 = Result()
        res4.set_verdict("inconclusive", 4, 5)
        resultlist.append(res4)
        self.assertTrue(resultlist.clean_inconcs())

        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        res.set_verdict(verdict="pass", retcode=0, duration=0)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="inconclusive", retcode=1, duration=5)
        res2.retries_left = 1
        res3 = Result()
        res3.set_verdict("pass", 4, 5)
        res3.retries_left = 0
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        self.assertFalse(resultlist.clean_inconcs())

    def test_pass_rate(self):
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        res.set_verdict(verdict="pass", retcode=0, duration=0)
        dictionary = {"retcode": 1}
        res2 = Result(kwargs=dictionary)
        res2.set_verdict(verdict="fail", retcode=1, duration=5)
        res3 = Result()
        res3.set_verdict("inconclusive", 4, 5)
        res4 = Result(kwargs=dictionary)
        res4.set_verdict(verdict="skip", retcode=1, duration=5)
        resultlist = ResultList()
        resultlist.append(res)
        resultlist.append(res2)
        resultlist.append(res3)
        self.assertEquals(resultlist.pass_rate(), "50.00 %")
        self.assertEquals(resultlist.pass_rate(include_inconclusive=True), "33.33 %")
        self.assertEquals(resultlist.pass_rate(include_skips=True), "50.00 %")
        resultlist.append(res4)
        self.assertEquals(resultlist.pass_rate(include_skips=True, include_inconclusive=True),
                          "25.00 %")


class ResultTestcase(unittest.TestCase):

    def setUp(self):
        self.args_tc = argparse.Namespace(
            available=False, version=False, bin=None, binary=False, channel=None,
            clean=False, cloud=False, component=False, device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False, iface=None, kill_putty=False, list=False,
            listsuites=False, log='./log', my_duts=None, nobuf=None, pause_when_external_dut=False,
            putty=False, reset=False, silent=True, skip_case=False,
            skip_rampdown=False, skip_rampup=False, platform=None,
            status=False, suite=None, tc="test_cmdline", tc_cfg=None,
            tcdir="examples", testtype=False, type="process",
            subtype=None, use_sniffer=False, valgrind=False,
            valgrind_tool=None, verbose=False, repeat=0, feature=None,
            suitedir="./test/suites", forceflash_once=True, forceflash=False,
            stop_on_failure=False, json=False)

    def test_init(self):
        dictionary = {"retcode": 0}
        res = Result(kwargs=dictionary)
        self.assertEqual(res.get_verdict(), "pass")

        dictionary = {"retcode": 1}
        res = Result(kwargs=dictionary)
        self.assertEqual(res.get_verdict(), "fail")

    def test_set_verdict(self):
        result = Result()
        result.set_verdict("pass", 0, 10)
        self.assertEqual(result.get_verdict(), "pass")
        self.assertEqual(result.retcode, 0)
        self.assertEqual(result.get_duration(), '0:00:10')
        self.assertEqual(result.get_duration(True), '10')

        with self.assertRaises(ValueError):
            result.set_verdict("wat")

    def test_haslogs(self):
        result = Result()
        result.logpath = os.path.join(os.path.dirname(__file__), "test_logpath")
        files = result.has_logs()
        self.assertTrue(files)
        self.assertEqual(len(files), 2)

        result = Result()
        result.logpath = None
        files = result.has_logs()
        self.assertListEqual(files, [])

    def test_result_metainfo_generation(self):
        pass_result = Result()
        pass_result.set_verdict('pass', 0, 10)
        dinfo = DutInformation("Test_platform", "123456", "1")
        dinfo.build = Build(ref="test_file", type="file")
        pass_result.add_dutinformation(dinfo)
        self.args_tc.branch = "test_branch"
        self.args_tc.commitId = "123456"
        self.args_tc.gitUrl = "url"
        self.args_tc.buildUrl = "url2"
        self.args_tc.campaign = "campaign"
        self.args_tc.jobId = "test_job"
        self.args_tc.toolchain = "toolchain"
        self.args_tc.buildDate = "today"
        pass_result.build_result_metadata(args=self.args_tc)
        self.assertEqual(pass_result.build_branch, "test_branch")
        self.assertEqual(pass_result.buildcommit, "123456")
        self.assertEqual(pass_result.build_git_url, "url")
        self.assertEqual(pass_result.build_url, "url2")
        self.assertEqual(pass_result.campaign, "campaign")
        self.assertEqual(pass_result.job_id, "test_job")
        self.assertEqual(pass_result.toolchain, "toolchain")
        self.assertEqual(pass_result.build_date, "today")


if __name__ == '__main__':
    unittest.main()
